@@ -97,7 +97,10 @@ class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
         else:
             return False
 
-    def __call__(self, func: Callable) -> Callable:
+    def __call__(
+        self,
+        func: Callable[..., None],
+    ) -> Callable[..., None]:
         """Filter calls to the decorated function.
 
         Gets called when a function should be decorated.
@@ -105,7 +108,9 @@ class change_filter:  # noqa: N801,N806 pylint: disable=invalid-name
         Adds a filter which returns if we're not interested in the change-event
         and calls the wrapped function if we are.
 
-        We assume the function passed doesn't take any parameters.
+        We assume the function passed doesn't take any parameters. However, it
+        could take a "self" argument, so we can't cleary express this in the
+        type above.
 
         Args:
             func: The function to be decorated.
@@ -173,6 +178,8 @@ class KeyConfig:
         result = results[0]
         if result.cmd.name != "set-cmd-text":
             return cmdline
+        if not result.args:
+            return None  # doesn't look like this sets a command
         *flags, cmd = result.args
         if "-a" in flags or "--append" in flags or not cmd.startswith(":"):
             return None  # doesn't look like this sets a command
@@ -307,7 +314,7 @@ class Config(QObject):
 
     def _init_values(self) -> None:
         """Populate the self._values dict."""
-        self._values: Mapping = {}
+        self._values: Mapping[str, configutils.Values] = {}
         for name, opt in configdata.DATA.items():
             self._values[name] = configutils.Values(opt)
 
@@ -380,6 +387,8 @@ class Config(QObject):
         """Get the given setting converted for Python code.
 
         Args:
+            name: The name of the setting to get.
+            url: The URL to get the value for.
             fallback: Use the global value if there's no URL-specific one.
         """
         opt = self.get_opt(name)
@@ -540,11 +549,12 @@ class Config(QObject):
         Here, we check all those saved copies for mutations, and if something
         mutated, we call set_obj again so we save the new value.
         """
-        for name, (old_value, new_value) in self._mutables.items():
+        mutables = self._mutables.items()
+        self._mutables = {}
+        for name, (old_value, new_value) in mutables:
             if old_value != new_value:
                 log.config.debug("{} was mutated, updating".format(name))
                 self.set_obj(name, new_value, save_yaml=save_yaml)
-        self._mutables = {}
 
     def dump_userconfig(self) -> str:
         """Get the part of the config which was changed by the user.
@@ -591,13 +601,13 @@ class ConfigContainer:
                               pattern=self._pattern)
 
     @contextlib.contextmanager
-    def _handle_error(self, action: str, name: str) -> Iterator[None]:
+    def _handle_error(self, action: str) -> Iterator[None]:
         try:
             yield
         except configexc.Error as e:
             if self._configapi is None:
                 raise
-            text = "While {} '{}'".format(action, name)
+            text = f"While {action}"
             self._configapi.errors.append(configexc.ConfigErrorDesc(text, e))
 
     def _with_prefix(self, prefix: str) -> 'ConfigContainer':
@@ -625,7 +635,7 @@ class ConfigContainer:
         if configdata.is_valid_prefix(name):
             return self._with_prefix(name)
 
-        with self._handle_error('getting', name):
+        with self._handle_error(f"getting '{name}'"):
             if self._configapi is None:
                 # access from Python code
                 return self._config.get(name)
@@ -648,7 +658,7 @@ class ConfigContainer:
             return
 
         name = self._join(attr)
-        with self._handle_error('setting', name):
+        with self._handle_error(f"setting '{name}'"):
             self._config.set_obj(name, value, pattern=self._pattern)
 
     def _join(self, attr: str) -> str:
