@@ -30,6 +30,7 @@ import os.path
 import shutil
 import venv as pyvenv
 import subprocess
+import platform
 from typing import List, Optional, Tuple, Dict, Union
 
 sys.path.insert(0, os.path.join(os.path.dirname(__file__), os.pardir))
@@ -224,6 +225,23 @@ def install_pyqt_binary(venv_dir: pathlib.Path, version: str) -> None:
     utils.print_title("Installing PyQt from binary")
     utils.print_col("No proprietary codec support will be available in "
                     "qutebrowser.", 'bold')
+
+    supported_archs = {
+        'linux': {'x86_64'},
+        'win32': {'x86', 'AMD64'},
+        'darwin': {'x86_64'},
+    }
+    if sys.platform not in supported_archs:
+        utils.print_error(f"{sys.platform} is not a supported platform by PyQt5 binary "
+                          "packages, this will most likely fail.")
+    elif platform.machine() not in supported_archs[sys.platform]:
+        utils.print_error(
+            f"{platform.machine()} is not a supported architecture for PyQt5 binaries "
+            f"on {sys.platform}, this will most likely fail.")
+    elif sys.platform == 'linux' and platform.libc_ver()[0] != 'glibc':
+        utils.print_error("Non-glibc Linux is not a supported platform for PyQt5 "
+                          "binaries, this will most likely fail.")
+
     pip_install(venv_dir, '-r', pyqt_requirements_file(version),
                 '--only-binary', 'PyQt5,PyQtWebEngine')
 
@@ -335,7 +353,7 @@ def _find_libs() -> Dict[Tuple[str, str], List[str]]:
     for line in ldconfig_proc.stdout.splitlines():
         match = pattern.fullmatch(line.strip())
         if match is None:
-            if 'libs found in cache' not in line:
+            if 'libs found in cache' not in line and 'Cache generated by:' not in line:
                 utils.print_col(f'Failed to match ldconfig output: {line}', 'yellow')
             continue
 
@@ -391,7 +409,13 @@ def install_dev_requirements(venv_dir: pathlib.Path) -> None:
     utils.print_title("Installing dev dependencies")
     pip_install(venv_dir,
                 '-r', str(requirements_file('dev')),
-                '-r', requirements_file('tests'))
+                '-r', str(requirements_file('check-manifest')),
+                '-r', str(requirements_file('flake8')),
+                '-r', str(requirements_file('mypy')),
+                '-r', str(requirements_file('pyroma')),
+                '-r', str(requirements_file('vulture')),
+                '-r', str(requirements_file('yamllint')),
+                '-r', str(requirements_file('tests')))
 
 
 def install_qutebrowser(venv_dir: pathlib.Path) -> None:
