@@ -19,8 +19,7 @@
 
 """Test hints based on html files with special comments."""
 
-import os
-import os.path
+import pathlib
 import textwrap
 import dataclasses
 from typing import Optional
@@ -32,16 +31,16 @@ from qutebrowser.utils import utils
 
 
 def collect_tests():
-    basedir = os.path.dirname(__file__)
-    datadir = os.path.join(basedir, 'data', 'hints', 'html')
-    files = [f for f in os.listdir(datadir) if f != 'README.md']
+    basedir = pathlib.Path(__file__).parent
+    datadir = basedir / 'data' / 'hints' / 'html'
+    files = [f.name for f in datadir.iterdir() if f.name != 'README.md']
     return files
 
 
 @dataclasses.dataclass
 class ParsedFile:
 
-    target: str
+    target: Optional[str]
     qtwebengine_todo: Optional[str]
 
 
@@ -55,9 +54,9 @@ class InvalidFile(Exception):
 
 def _parse_file(test_name):
     """Parse the given HTML file."""
-    file_path = os.path.join(os.path.abspath(os.path.dirname(__file__)),
-                             'data', 'hints', 'html', test_name)
-    with open(file_path, 'r', encoding='utf-8') as html:
+    file_path = (pathlib.Path(__file__).parent.resolve()
+                 / 'data' / 'hints' / 'html' / test_name)
+    with file_path.open('r', encoding='utf-8') as html:
         soup = bs4.BeautifulSoup(html, 'html.parser')
 
     comment = str(soup.find(text=lambda text: isinstance(text, bs4.Comment)))
@@ -108,11 +107,18 @@ def test_hints(test_name, zoom_text_only, zoom_level, find_implementation,
         quteproc.set_setting('zoom.text_only', str(zoom_text_only))
         quteproc.set_setting('hints.find_implementation', find_implementation)
     quteproc.send_cmd(':zoom {}'.format(zoom_level))
+
     # follow hint
     quteproc.send_cmd(':hint all normal')
-    quteproc.wait_for(message='hints: a', category='hints')
-    quteproc.send_cmd(':hint-follow a')
-    quteproc.wait_for_load_finished('data/' + parsed.target)
+
+    if parsed.target is None:
+        msg = quteproc.wait_for(message='No elements found.', category='message')
+        msg.expected = True
+    else:
+        quteproc.wait_for(message='hints: a', category='hints')
+        quteproc.send_cmd(':hint-follow a')
+        quteproc.wait_for_load_finished('data/' + parsed.target)
+
     # reset
     quteproc.send_cmd(':zoom 100')
     if not request.config.webengine:
@@ -121,9 +127,9 @@ def test_hints(test_name, zoom_text_only, zoom_level, find_implementation,
 
 
 @pytest.mark.skip  # Too flaky
-def test_word_hints_issue1393(quteproc, tmpdir):
-    dict_file = tmpdir / 'dict'
-    dict_file.write(textwrap.dedent("""
+def test_word_hints_issue1393(quteproc, tmp_path):
+    dict_file = tmp_path / 'dict'
+    dict_file.write_text(textwrap.dedent("""
         alph
         beta
         gamm
