@@ -44,6 +44,21 @@ def test_no_content_disposition(stubs, url, expected):
     assert filename == expected
 
 
+@pytest.mark.parametrize('value', [
+    # https://github.com/python/cpython/issues/87112
+    'inline; 0*²'.encode("iso-8859-1"),
+    # https://github.com/python/cpython/issues/81672
+    b'"',
+    # https://github.com/python/cpython/issues/93010
+    b'attachment; 0*00="foo"',
+    # FIXME: Should probably have more tests if this is still relevant after
+    # dropping QtWebKit.
+])
+def test_parse_content_disposition_invalid(value):
+    with pytest.raises(http.ContentDispositionError):
+        http.ContentDisposition.parse(value)
+
+
 @pytest.mark.parametrize('template', [
     '{}',
     'attachment; filename="{}"',
