@@ -76,7 +76,7 @@ def _check_completions(model, expected):
     assert sum(model.column_widths) == 100
 
 
-@pytest.fixture()
+@pytest.fixture
 def cmdutils_stub(monkeypatch, stubs):
     """Patch the cmdutils module to provide fake commands."""
     return monkeypatch.setattr(objects, 'commands', {
@@ -93,7 +93,7 @@ def cmdutils_stub(monkeypatch, stubs):
     })
 
 
-@pytest.fixture()
+@pytest.fixture
 def configdata_stub(config_stub, monkeypatch, configdata_init):
     """Patch the configdata module to provide fake data."""
     monkeypatch.setattr(configdata, 'DATA', collections.OrderedDict([
@@ -462,8 +462,7 @@ def test_filesystem_completion_model_interface(info, local_files_path):
 @hypothesis.given(
     as_uri=hst.booleans(),
     add_sep=hst.booleans(),
-    text=hst.text(alphabet=hst.characters(
-        blacklist_categories=['Cc'], blacklist_characters='\x00')),
+    text=hst.text(),
 )
 def test_filesystem_completion_hypothesis(info, as_uri, add_sep, text):
     if as_uri:
@@ -475,6 +474,12 @@ def test_filesystem_completion_hypothesis(info, as_uri, add_sep, text):
     model.set_pattern(text)
 
 
+@pytest.mark.parametrize('text', ['~\ud800', '~\x00'])
+def test_filesystem_completion_corner_cases(info, text):
+    model = filepathcategory.FilePathCategory('filepaths')
+    model.set_pattern(text)
+
+
 def test_default_filesystem_completion(qtmodeltester, config_stub, info,
                                        web_history_populated, quickmarks, bookmarks,
                                        local_files_path):
@@ -862,6 +867,34 @@ def test_tab_completion_delete(qtmodeltester, fake_web_tab, win_registry,
                       QUrl('https://duckduckgo.com')]
 
 
+def test_tab_focus_completion_delete(qtmodeltester, fake_web_tab, win_registry,
+                              tabbed_browser_stubs, info):
+    """Verify closing a tab by deleting it from the completion widget."""
+    tabbed_browser_stubs[0].widget.tabs = [
+        fake_web_tab(QUrl('https://github.com'), 'GitHub', 0),
+        fake_web_tab(QUrl('https://wikipedia.org'), 'Wikipedia', 1),
+        fake_web_tab(QUrl('https://duckduckgo.com'), 'DuckDuckGo', 2)
+    ]
+    tabbed_browser_stubs[1].widget.tabs = [
+        fake_web_tab(QUrl('https://wiki.archlinux.org'), 'ArchWiki', 0),
+    ]
+    model = miscmodels.tab_focus(info=info)
+    model.set_pattern('')
+    qtmodeltester.check(model)
+
+    parent = model.index(0, 0)
+    idx = model.index(1, 0, parent)
+
+    # sanity checks
+    assert model.data(parent) == "Tabs"
+    assert model.data(idx) == '2'
+
+    model.delete_cur_item(idx)
+    actual = [tab.url() for tab in tabbed_browser_stubs[0].widget.tabs]
+    assert actual == [QUrl('https://github.com'),
+                      QUrl('https://duckduckgo.com')]
+
+
 def test_tab_completion_not_sorted(qtmodeltester, fake_web_tab, win_registry,
                                    tabbed_browser_stubs):
     """Ensure that the completion row order is the same as tab index order.
