@@ -319,12 +319,13 @@ class TestAdd:
                 match=":config-list-add can only be used for lists"):
             commands.config_list_add('history_gap_interval', 'value')
 
-    @pytest.mark.parametrize('value', ['', None, 42])
-    def test_list_add_invalid_values(self, commands, value):
-        with pytest.raises(
-                cmdutils.CommandError,
-                match="Invalid value '{}'".format(value)):
-            commands.config_list_add('content.blocking.whitelist', value)
+    def test_list_add_invalid_value(self, commands):
+        with pytest.raises(cmdutils.CommandError, match="Invalid value ''"):
+            commands.config_list_add('content.blocking.whitelist', '')
+
+    # FIXME test value conversion for :list-add like in test_dict_add_value_type
+    # (once we have a List config option using a non-str type, or a way to
+    # dynamically add new option definitions).
 
     @pytest.mark.parametrize('value', ['test1', 'test2'])
     @pytest.mark.parametrize('temp', [True, False])
@@ -368,11 +369,9 @@ class TestAdd:
                 match=":config-dict-add can only be used for dicts"):
             commands.config_dict_add('history_gap_interval', 'key', 'value')
 
-    @pytest.mark.parametrize('value', ['', None, 42])
-    def test_dict_add_invalid_values(self, commands, value):
-        with pytest.raises(cmdutils.CommandError,
-                           match="Invalid value '{}'".format(value)):
-            commands.config_dict_add('aliases', 'missingkey', value)
+    def test_dict_add_invalid_value(self, commands):
+        with pytest.raises(cmdutils.CommandError, match="Invalid value ''"):
+            commands.config_dict_add('aliases', 'missingkey', '')
 
 
 class TestRemove:
@@ -407,8 +406,12 @@ class TestRemove:
     def test_list_remove_no_value(self, commands):
         with pytest.raises(
                 cmdutils.CommandError,
-                match="never is not in colors.completion.fg!"):
-            commands.config_list_remove('colors.completion.fg', 'never')
+                match="#133742 is not in colors.completion.fg!"):
+            commands.config_list_remove('colors.completion.fg', '#133742')
+
+    # FIXME test value conversion for :list-remove like in test_dict_add_value_type
+    # (once we have a List config option using a non-str type, or a way to
+    # dynamically add new option definitions).
 
     @pytest.mark.parametrize('key', ['w', 'q'])
     @pytest.mark.parametrize('temp', [True, False])
@@ -575,6 +578,19 @@ class TestSource:
                     " division by zero")
         assert str(excinfo.value) == expected
 
+    def test_invalid_mutable(self, commands, config_tmpdir):
+        pyfile = config_tmpdir / 'config.py'
+        src = 'c.url.searchengines["maps"] = "https://www.google.com/maps?q=%s"'
+        pyfile.write_text(src, encoding="utf-8")
+
+        with pytest.raises(cmdutils.CommandError) as excinfo:
+            commands.config_source()
+
+        err = "Invalid value 'https://www.google.com/maps?q=%s' - must contain \"{}\""
+        expected = ("Errors occurred while reading config.py:\n"
+                    f"  While updating mutated values: {err}")
+        assert str(excinfo.value) == expected
+
 
 class TestEdit:
 
