@@ -131,6 +131,20 @@ def test_no_name_with_fallback():
     assert script.name == r"C:\COM1"
 
 
+@pytest.mark.parametrize('properties, inc_counter, expected', [
+    ([("name", "gorilla")], False, "GM-gorilla"),
+    ([("namespace", "apes"), ("name", "gorilla")], False, "GM-apes/gorilla"),
+
+    ([("name", "gorilla")], True, "GM-gorilla-2"),
+    ([("namespace", "apes"), ("name", "gorilla")], True, "GM-apes/gorilla-2"),
+])
+def test_full_name(properties, inc_counter, expected):
+    script = greasemonkey.GreasemonkeyScript(properties, code="")
+    if inc_counter:
+        script.dedup_suffix += 1
+    assert script.full_name() == expected
+
+
 def test_bad_scheme(caplog):
     """qute:// isn't in the list of allowed schemes."""
     _save_script("var nothing = true;\n", 'nothing.user.js')
@@ -198,7 +212,7 @@ class TestForceDocumentEnd:
         assert script.needs_document_end_workaround() == force
 
 
-def test_required_scripts_are_included(download_stub, tmpdir):
+def test_required_scripts_are_included(download_stub, tmp_path):
     test_require_script = textwrap.dedent("""
         // ==UserScript==
         // @name qutebrowser test userscript
@@ -212,7 +226,7 @@ def test_required_scripts_are_included(download_stub, tmpdir):
         console.log("Script is running.");
     """)
     _save_script(test_require_script, 'requiring.user.js')
-    (tmpdir / 'test.js').write_text('REQUIRED SCRIPT', encoding='UTF-8')
+    (tmp_path / 'test.js').write_text('REQUIRED SCRIPT', encoding='UTF-8')
 
     gm_manager = greasemonkey.GreasemonkeyManager()
     assert len(gm_manager._in_progress_dls) == 1
