@@ -61,6 +61,12 @@ from qutebrowser.utils import usertypes
         {'0': 'a', '1': 'b'},
         id='object-fromentries',
     ),
+    pytest.param(
+        QUrl("https://test.qutebrowser.org/linkedin"),
+        '[1, 2, 3].at(1)',
+        2,
+        id='array-at',
+    ),
 ])
 def test_js_quirks(config_stub, js_tester_webengine, base_url, source, expected):
     config_stub.val.content.site_specific_quirks.skip = []
