@@ -21,6 +21,7 @@
 
 from PyQt5.QtCore import QBuffer
 
+from qutebrowser.misc import objects
 from qutebrowser import app
 
 
@@ -30,7 +31,7 @@ def test_on_focus_changed_issue1484(monkeypatch, qapp, caplog):
     For some reason, Qt sometimes calls on_focus_changed() with a QBuffer as
     argument. Let's make sure we handle that gracefully.
     """
-    monkeypatch.setattr(app, 'q_app', qapp)
+    monkeypatch.setattr(objects, 'qapp', qapp)
 
     buf = QBuffer()
     app.on_focus_changed(buf, buf)
